package forktest;

import java.util.ArrayList;
import java.util.List;

import junit.framework.TestCase;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jbpm.JbpmConfiguration;
import org.jbpm.JbpmContext;
import org.jbpm.graph.def.ProcessDefinition;
import org.jbpm.graph.exe.ProcessInstance;
import org.jbpm.graph.exe.Token;
import org.jbpm.persistence.db.DbPersistenceServiceFactory;
import org.jbpm.svc.Services;

public class TokenSubscriberUnitTest extends TestCase implements JbpmStateManagerSubscriber
{
	/**
	 * Some helpers
	 */
	private Log log = LogFactory.getLog(TokenSubscriberUnitTest.class);
	
	private static JbpmConfiguration jbpmConfiguration = JbpmConfiguration.getInstance();
	private static DbPersistenceServiceFactory dbPersistenceServiceFactory = (DbPersistenceServiceFactory) jbpmConfiguration.getServiceFactory(Services.SERVICENAME_PERSISTENCE);

	private static JbpmContext jbpmContext;
	private static JbpmStateManager jsm = JbpmStateManager.getInstance();

	public TokenSubscriberUnitTest()
	{
	}

	/**
	 * Set up the basic in-memory DB, deploy the process description and
	 * set up a jBPM context.
	 * @see jBPM docs wrt jbpm configuration file
	 */
	protected void setUp() throws Exception
	{
		super.setUp();
		dbPersistenceServiceFactory.createSchema();
		deployProcess();
		
		jbpmConfiguration.startJobExecutor();
	}

	/**
	 * Tidy up.
	 */
	protected void tearDown() throws Exception
	{
		super.tearDown();
		dbPersistenceServiceFactory.dropSchema();
		jbpmContext = null;
	}

	/**
	 * Get the process description file, parse it and deploy the
	 * process definition.
	 */
	private void deployProcess() 
	{
		JbpmContext jbpmContext = jbpmConfiguration.createJbpmContext();
		try 
		{
			ProcessDefinition processDefinition = 
				ProcessDefinition.parseXmlResource("ForkProcess/processdefinition.xml");
			jbpmContext.deployProcessDefinition(processDefinition);
		} finally 
		{
			jbpmContext.close();
		}
	}

	private List<Long> tkn = new ArrayList<Long>();

	public void testFork()
	{
		// log4j.logger.org.hibernate.engine.StatefulPersistenceContext.ProxyWarnLog=ERROR
		log.info("Starting: " + this);
		String end_state = "illegal";
		
		jsm.subscribe(this);
		
		jbpmContext = jbpmConfiguration.createJbpmContext();
		
		ProcessInstance	processInstance = jbpmContext.newProcessInstanceForUpdate("ForkTest");
		long processInstanceId = processInstance.getId();
		
		boolean active_process = false;
		
		try 
		{
			processInstance.getContextInstance().setVariable("choice", "OptionA");
			processInstance.getTaskMgmtInstance().createStartTaskInstance();
			jbpmContext.save(processInstance);
		} finally 
		{
			jbpmContext.close();
			active_process = true;
		}

		while (active_process)
		{
			try
			{
				Thread.sleep(1000);
			}
			catch (InterruptedException e)
			{
				e.printStackTrace();
			}
			
			jbpmContext = jbpmConfiguration.createJbpmContext();
			
			try 
			{
				processInstance = jbpmContext.loadProcessInstance(processInstanceId);

				Token root = processInstance.getRootToken();
				
				if (root.hasEnded())
				{
					end_state = root.getNode().getName();
					active_process = false;
				}
				else
				if (tkn.size() != 0)
				{
					long tid = tkn.get(0).longValue();
					Token token = jbpmContext.getTokenForUpdate(tid);
					String name = token.getFullName();

					if (! token.isLocked())
					{
						log.info("Signalling token: " + name);
						tkn.remove(0);
						token.signal();
					}
					else
					{
						log.info("Token : " + name + " is locked");
					}
				}
//				jbpmContext.save(processInstance);
			} finally 
			{
				jbpmContext.close();
			}
		}

		assertEquals("end", end_state);
	}

	public void enterState(String state, Token token)
	{
		log.info("System state: " + state);
		tkn.add(new Long(token.getId()));
	}

}
